@@ -22,7 +22,7 @@ import configparser
 from pathlib import Path
 import tempfile
 
-import notmuch
+import notmuch2
 from .remote import Remote
 
 class Local:
@@ -31,7 +31,7 @@ class Local:
 
 
   # NOTE: Update README when changing this map.
-  translate_labels = {
+  translate_labels_default = {
                       'INBOX'     : 'inbox',
                       'SPAM'      : 'spam',
                       'TRASH'     : 'trash',
@@ -49,7 +49,7 @@ class Local:
                       'CATEGORY_FORUMS'       : 'forums',
                       }
 
-  labels_translate = { v: k for k, v in translate_labels.items () }
+  labels_translate_default = { v: k for k, v in translate_labels_default.items () }
 
   ignore_labels = set ([
                         'archive',
@@ -66,6 +66,35 @@ class Local:
                         'voicemail',
                         ])
 
+  def update_translation(self, remote, local):
+    """
+    Convenience function to ensure both maps (remote -> local and local -> remote)
+    get updated when you update a translation.
+    """
+    # Did you reverse the parameters?
+    assert remote in self.translate_labels
+    self.translate_labels[remote] = local
+    self.labels_translate = { v: k for k, v in self.translate_labels.items () }
+
+  def update_translation_list_with_overlay(self, translation_list_overlay):
+    """
+    Takes a list with an even number of items. The list is interpreted as a list of pairs
+    of (remote, local), where each member of each pair is a string. Each pair is added to the
+    translation, overwriting the translation if one already exists (in either direction).
+    If either the remote or the local labels are non-unique, the later items in the list will
+    overwrite the earlier ones in the direction in which the source is non-unique (for example,
+    ["a", "1", "b", 2", "a", "3"] will yield {'a': 3, 'b': 2} in one direction and {1: 'a', 2: 'b', 3: 'a'}
+    in the other).
+    """
+
+    if len(translation_list_overlay) % 2 != 0:
+      raise Exception(f'Translation list overlay must have an even number of items: {translation_list_overlay}')
+
+    for i in range(0,len(translation_list_overlay),2):
+      (remote, local) = translation_list_overlay[i], translation_list_overlay[i+1]
+      self.translate_labels[remote] = local
+      self.labels_translate[local] = remote
+
   class RepositoryException (Exception):
     pass
 
@@ -80,6 +109,8 @@ class Local:
     ignore_remote_labels = None
     remove_local_messages = True
     file_extension = None
+    local_trash_tag = 'trash'
+    translation_list_overlay = None
 
     def __init__ (self, config_f):
       self.config_f = config_f
@@ -103,6 +134,8 @@ class Local:
       self.ignore_tags = set(self.json.get ('ignore_tags', []))
       self.ignore_remote_labels = set(self.json.get ('ignore_remote_labels', Remote.DEFAULT_IGNORE_LABELS))
       self.file_extension = self.json.get ('file_extension', '')
+      self.local_trash_tag = self.json.get ('local_trash_tag', 'trash')
+      self.translation_list_overlay = self.json.get ('translation_list_overlay', [])
 
     def write (self):
       self.json = {}
@@ -116,6 +149,8 @@ class Local:
       self.json['ignore_remote_labels'] = list(self.ignore_remote_labels)
       self.json['remove_local_messages'] = self.remove_local_messages
       self.json['file_extension'] = self.file_extension
+      self.json['local_trash_tag'] = self.local_trash_tag
+      self.json['translation_list_overlay'] = self.translation_list_overlay
 
       if os.path.exists (self.config_f):
         shutil.copyfile (self.config_f, self.config_f + '.bak')
@@ -166,7 +201,7 @@ class Local:
 
     def set_file_extension (self, t):
       try:
-        with tempfile.NamedTemporaryFile (dir = os.path.dirname (self.state_f), suffix = t) as fd:
+        with tempfile.NamedTemporaryFile (dir = os.path.dirname (self.config_f), suffix = t) as _:
           pass
 
         self.file_extension = t.strip ()
@@ -175,6 +210,23 @@ class Local:
         print ("Failed creating test file with file extension: " + t + ", not set.")
         raise
 
+    def set_local_trash_tag (self, t):
+      if ',' in t:
+        print('The local_trash_tag must be a single tag, not a list.  Commas are not allowed.')
+        raise ValueError()
+      self.local_trash_tag = t.strip() or 'trash'
+      self.write()
+
+    def set_translation_list_overlay (self, t):
+      if len(t.strip ()) == 0:
+        self.translation_list_overlay = []
+      else:
+        self.translation_list_overlay = [ tt.strip () for tt in t.split(',') ]
+      if len(self.translation_list_overlay) % 2 != 0:
+        raise Exception(f'Translation list overlay must have an even number of items: {self.translation_list_overlay}')
+      self.write ()
+
+
 
   class State:
     # last historyid of last synchronized message, anything that has happened
@@ -241,7 +293,7 @@ class Local:
       self.lastmod = m
       self.write ()
 
-
+  # we are in the class "Local"; this is the Local instance constructor
   def __init__ (self, g):
     self.gmailieer = g
     self.wd = os.getcwd ()
@@ -255,6 +307,10 @@ class Local:
     # mail store
     self.md = os.path.join (self.wd, 'mail')
 
+    # initialize label translation instance variables
+    self.translate_labels = Local.translate_labels_default.copy()
+    self.labels_translate = Local.labels_translate_default.copy()
+
   def load_repository (self, block = False):
     """
     Loads the current local repository
@@ -269,25 +325,13 @@ class Local:
              for mail_dir in ('cur', 'new', 'tmp')]):
       raise Local.RepositoryException ('local repository not initialized: could not find mail dir structure')
 
-    self.config = Local.Config (self.config_f)
-    self.state = Local.State (self.state_f, self.config)
-
-    self.ignore_labels = self.ignore_labels | self.config.ignore_tags
-
     ## Check if we are in the notmuch db
-    with notmuch.Database () as db:
+    with notmuch2.Database () as db:
       try:
-        self.nm_dir  = db.get_directory (os.path.abspath(self.md))
-        if self.nm_dir is not None:
-          self.nm_dir = self.nm_dir.path
-        else:
-          # probably empty dir
-          self.nm_dir = os.path.abspath (self.md)
-
-        self.nm_relative = self.nm_dir[len(db.get_path ())+1:]
-
-      except notmuch.errors.FileError:
+        self.nm_relative=str(Path(self.md).relative_to(db.path))
+      except ValueError:
         raise Local.RepositoryException ("local mail repository not in notmuch db")
+      self.nm_dir=str(Path(self.md).resolve())
 
     ## Lock repository
     try:
@@ -299,17 +343,19 @@ class Local:
     except OSError:
       raise Local.RepositoryException ("failed to lock repository (probably in use by another gmi instance)")
 
+    self.config = Local.Config (self.config_f)
+    self.state = Local.State (self.state_f, self.config)
+
+    self.ignore_labels = self.ignore_labels | self.config.ignore_tags
+    self.update_translation('TRASH', self.config.local_trash_tag)
+    self.update_translation_list_with_overlay(self.config.translation_list_overlay)
+
     self.__load_cache__ ()
 
     # load notmuch config
-    cfg = os.environ.get('NOTMUCH_CONFIG', os.path.expanduser('~/.notmuch-config'))
-    if not os.path.exists (cfg):
-      raise Local.RepositoryException("could not find notmuch-config: %s" % cfg)
-
-    self.nmconfig = configparser.ConfigParser ()
-    self.nmconfig.read (cfg)
-    self.new_tags = self.nmconfig['new']['tags'].split (';')
-    self.new_tags = [t.strip () for t in self.new_tags if len(t.strip()) > 0]
+    with notmuch2.Database() as db:
+      self.new_tags = db.config.get("new.tags", "").split(';')
+    self.new_tags = [t.strip() for t in self.new_tags if len(t.strip()) > 0]
 
     self.loaded = True
 
@@ -319,12 +365,12 @@ class Local:
     ## this cache is used to know which messages we have a physical copy of.
     ## hopefully this won't grow too gigantic with lots of messages.
     self.files = []
-    for (dp, dirnames, fnames) in os.walk (os.path.join (self.md, 'cur')):
+    for (_, _, fnames) in os.walk (os.path.join (self.md, 'cur')):
       _fnames = ( 'cur/' + f for f in fnames )
       self.files.extend (_fnames)
       break
 
-    for (dp, dirnames, fnames) in os.walk (os.path.join (self.md, 'new')):
+    for (_, _, fnames) in os.walk (os.path.join (self.md, 'new')):
       _fnames = ( 'new/' + f for f in fnames )
       self.files.extend (_fnames)
       break
@@ -370,7 +416,7 @@ class Local:
     """
     Update cache with filenames from nmsg, removing the old:
 
-      nmsg - NotmuchMessage
+      nmsg - notmuch2.Message
       old  - tuple of old gid and old fname
     """
 
@@ -383,7 +429,8 @@ class Local:
       self.gids.pop (old_gid)
 
     # add message to cache
-    for _f in nmsg.get_filenames ():
+    fname_iter = nmsg.filenames ()
+    for _f in fname_iter:
       if self.contains (_f):
         new_f = Path (_f)
 
@@ -404,7 +451,7 @@ class Local:
     messages = []
 
     for m in msgs:
-      for fname in m.get_filenames ():
+      for fname in m.filenames ():
         if self.contains (fname):
           # get gmail id
           gid = self.__filename_to_gid__ (os.path.basename (fname))
@@ -428,7 +475,7 @@ class Local:
       return None
 
   def __make_maildir_name__ (self, m, labels):
-    # http://cr.yp.to/proto/maildir.html
+    # https://cr.yp.to/proto/maildir.html
     ext = ''
     if self.config.file_extension:
         ext = '.' + self.config.file_extension
@@ -468,13 +515,16 @@ class Local:
       return
 
     fname = os.path.join (self.md, fname)
-    nmsg  = db.find_message_by_filename (fname)
+    try:
+        nmsg = db.get(fname)
+    except LookupError:
+        nmsg = None
 
     if self.dry_run:
       print ("(dry-run) deleting %s: %s." % (gid, fname))
     else:
       if nmsg is not None:
-        db.remove_message (fname)
+        db.remove(fname)
       os.unlink (fname)
 
       self.files.remove (ffname)
@@ -507,7 +557,7 @@ class Local:
       raise Local.RepositoryException ("local file already exists: %s" % p)
 
     if os.path.exists (tmp_p):
-      raise Local.RepositoryException ("local file already exists: %s" % p)
+      raise Local.RepositoryException ("local temporary file already exists: %s" % tmp_p)
 
     if not self.dry_run:
       with open (tmp_p, 'wb') as fd:
@@ -569,57 +619,52 @@ class Local:
         print ("done.")
 
         if not os.path.exists (fname):
-          raise Local.RepositoryException ("tried to update tags on non-existant file: %s" % fname)
+          raise Local.RepositoryException ("tried to update tags on non-existent file: %s" % fname)
 
       else:
-        print ("(dry-run) tried to update tags on non-existant file: %s" % fname)
+        print ("(dry-run) tried to update tags on non-existent file: %s" % fname)
 
-    nmsg  = db.find_message_by_filename (fname)
+    try:
+      nmsg = db.get(fname)
+    except LookupError:
+      nmsg = None
 
     if nmsg is None:
       if self.dry_run:
         print ("(dry-run) adding message: %s: %s, with tags: %s" % (gid, fname, str(labels)))
       else:
         try:
-          if hasattr (notmuch.Database, 'index_file'):
-            (nmsg, stat) = db.index_file (fname, True)
-          else:
-            (nmsg, stat) = db.add_message (fname, True)
-        except notmuch.errors.FileNotEmailError:
+          (nmsg, _) = db.add (fname, sync_flags = True)
+        except notmuch2.FileNotEmailError:
           print('%s is not an email' % fname)
           return True
-        nmsg.freeze ()
 
         # adding initial tags
-        for t in labels:
-          nmsg.add_tag (t, True)
+        with nmsg.frozen():
+          for t in labels:
+            nmsg.tags.add (t)
 
-        for t in self.new_tags:
-          nmsg.add_tag (t, True)
+          for t in self.new_tags:
+            nmsg.tags.add (t)
 
-        nmsg.thaw ()
-        nmsg.tags_to_maildir_flags ()
+        nmsg.tags.to_maildir_flags()
         self.__update_cache__ (nmsg)
 
       return True
 
     else:
       # message is already in db, set local tags to match remote tags
-      otags   = set(nmsg.get_tags ())
+      otags   = nmsg.tags
       igntags = otags & self.ignore_labels
       otags   = otags - self.ignore_labels # remove ignored tags while checking
       if otags != set (labels):
         labels.extend (igntags) # add back local ignored tags before adding
         if not self.dry_run:
-          nmsg.freeze ()
-
-          nmsg.remove_all_tags ()
-          for t in labels:
-            nmsg.add_tag (t, False)
-
-          nmsg.thaw ()
-
-          nmsg.tags_to_maildir_flags ()
+          with nmsg.frozen():
+            nmsg.tags.clear()
+            for t in labels:
+              nmsg.tags.add (t)
+          nmsg.tags.to_maildir_flags()
           self.__update_cache__ (nmsg, (gid, fname))
 
         else:
