@@ -333,16 +333,16 @@ class Remote:
       try:
         batch.execute (http = self.http)
 
-        # gradually reduce user delay if we had 10 ok batches
+        # gradually reduce user delay upon every ok batch
         user_rate_ok += 1
-        if user_rate_delay > 0 and user_rate_ok > 10:
+        if user_rate_delay > 0 and user_rate_ok > 0:
           user_rate_delay = user_rate_delay // 2
           print ("remote: decreasing delay to %s" % user_rate_delay)
           user_rate_ok    = 0
 
-        # gradually increase batch request size if we had 10 ok requests
+        # gradually increase batch request size upon every ok request
         req_ok += 1
-        if max_req < self.BATCH_REQUEST_SIZE and req_ok > 10:
+        if max_req < self.BATCH_REQUEST_SIZE and req_ok > 0:
           max_req = min (max_req * 2, self.BATCH_REQUEST_SIZE)
           print ("remote: increasing batch request size to: %d" % max_req)
           req_ok  = 0
@@ -488,8 +488,8 @@ class Remote:
     gid    = gmsg['id']
 
     found = False
-    for f in nmsg.get_filenames ():
-      if gid in f:
+    for f in nmsg.filenames():
+      if gid in str(f):
         found = True
 
     # this can happen if a draft is edited remotely and is synced before it is sent. we'll
@@ -528,7 +528,7 @@ class Remote:
     labels = set(labels)
 
     # current tags
-    tags = set(nmsg.get_tags ())
+    tags = nmsg.tags
 
     # remove special notmuch tags
     tags = tags - self.gmailieer.local.ignore_labels
@@ -606,7 +606,7 @@ class Remote:
     Push label changes
     """
     max_req = self.BATCH_REQUEST_SIZE
-    N       = len (actions)
+    N       = len(actions)
     i       = 0
     j       = 0
 
@@ -615,19 +615,19 @@ class Remote:
     # How many requests with the current delay returned ok.
     user_rate_ok        = 0
 
-    def _cb (rid, resp, excep):
+    def _cb(rid, resp, excep):
       nonlocal j
       if excep is not None:
         if type(excep) is googleapiclient.errors.HttpError and excep.resp.status == 404:
           # message could not be found this is probably a deleted message, spam or draft
           # message since these are not included in the messages.get() query by default.
-          print ("remote: could not find remote message: %s!" % gids[j])
+          print ("remote: could not find remote message: %s!" % resp)
           j += 1
           return
 
         elif type(excep) is googleapiclient.errors.HttpError and excep.resp.status == 400:
           # message id invalid, probably caused by stray files in the mail repo
-          print ("remote: message id: %s is invalid! are there any non-lieer files created in the lieer repository?" % gids[j])
+          print ("remote: message id is invalid! are there any non-lieer files created in the lieer repository? %s" % resp)
           j += 1
           return
 
@@ -639,16 +639,16 @@ class Remote:
       else:
         j += 1
 
-      cb (resp)
+      cb(resp)
 
     while i < N:
       n = 0
       j = i
-      batch = self.service.new_batch_http_request  (callback = _cb)
+      batch = self.service.new_batch_http_request(callback = _cb)
 
       while n < max_req and i < N:
         a = actions[i]
-        batch.add (a)
+        batch.add(a)
         n += 1
         i += 1
 
@@ -666,14 +666,14 @@ class Remote:
           user_rate_delay = user_rate_delay // 2
           user_rate_ok    = 0
 
-      except Remote.UserRateException as ex:
+      except Remote.UserRateException:
         user_rate_delay = user_rate_delay * 2 + 1
         print ("remote: user rate error, increasing delay to %s" % user_rate_delay)
         user_rate_ok = 0
 
         i = j # reset
 
-      except Remote.BatchException as ex:
+      except Remote.BatchException:
         if max_req > self.MIN_BATCH_REQUEST_SIZE:
           max_req = max_req / 2
           i = j # reset
